      subroutine RAP_all3 (mat3, rbuf, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.3    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.2    date   26-06-1997
c           accomodating the 19 pt stencil
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date 11-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   calculates RAP operator on all levels.
c   mklvlinfo must have been called previously.
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      double precision mat3 (*), rbuf (*)
      integer ibuf (*)

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat3      i    The nx x ny x nz x 27  coefficient matrix of the
c                    discrete Laplacian
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilevel, hilevel, aux1offs, aux2offs

c     aux1offs  Auxiliary offset in scratch space
c     aux2offs  Auxiliary offset in scratch space
c     corsnini  Ibuf (corsnini) contains the method of coarsening
c               for this block. (fullcors or semicors) (module
c               mgpreset)
c     flevel0   Flag to denote whether we are on level 0
c               (module mgpreset)
c     hilevel   Maximum level of coarse grids (level 0 is finest)
c     hileveli  Ibuf (hileveli) contains the highest level of
c               coarsening for this block. (<= MAXLEVEL) (module
c               mgpreset)
c     ilevel    Levelcounter (multi grid levels)
c     matri     Rbuf (ibuf (matri + ilevel)) is the first element
c               of the RAP operator at level ilevel (ilevel >= 1)
c               (module mgpreset)
c     offscri   Ibuf (offscri) contains index where scratch space begins
c               in rbuf (module mgpreset)
c     semicors  Semicoarsening (type coarsening) (module mgpreset)
c     sizexi    Ibuf(sizexi + ilevel)) contains number of points
c               in x direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizeyi    Ibuf(sizeyi + ilevel)) contains number of points
c               in y direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizezi    Ibuf(sizezi + ilevel)) contains number of points
c               in z direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     slacki    Ibuf (slacki) is the first element of the 6 element
c               slack array xslacklo, xslackhi, yslacklo, yslackhi,
c               zslacklo, zslackhi, (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     FREESCS3  Gives reserved scratch space back to buffer (MG3)
c     GETSCS3   Reserves a block of scratch space from buffer (MG3)
c     RAPX3     Forms coarse grid operator semic in x-direction (MG3)
c     RAPXYZ3   Forms coarse grid oper semic in x, y & z-direction (MG3)
c     RAPY3     Forms coarse grid operator semic in y-direction (MG3)
c     RAPZ3     Forms coarse grid operator semic in z-direction (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   trivial
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      hilevel = ibuf (hileveli)
      if (ibuf (corsnini).eq.SEMICORS) then

c--- Alternating RAPx, RAPy and RAPz
         call RAPx3 (mat3, rbuf ( ibuf( matri + 1)),
     +       ibuf (sizexi), ibuf (sizeyi), ibuf (sizezi),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))
        flevel0 = .false.
        do ilevel = 1, hilevel - 1
          if (mod (ilevel, 3).eq.1) then

c--- RAPy every third level + 1 (semicoarsening in y-direction)
             call RAPy3 (rbuf (ibuf (matri + ilevel)),
     +                  rbuf (ibuf (matri + ilevel + 1)),
     +                  ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +                  ibuf (sizezi + ilevel), 0, 0, 0, 0, 0, 0)
          else if (mod (ilevel, 3).eq.2) then

c--- RAPz every third level + 2 (semicoarsening in z-direction)
             call RAPz3 (rbuf (ibuf (matri + ilevel)),
     +                  rbuf (ibuf (matri + ilevel + 1)),
     +                  ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +                  ibuf (sizezi + ilevel), 0, 0, 0, 0, 0, 0)
          else

c--- RAPx on even levels (semicoarsening in x-direction)
             call RAPx3 (rbuf (ibuf (matri + ilevel)),
     +                  rbuf (ibuf (matri + ilevel + 1)),
     +                  ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +                  ibuf (sizezi + ilevel), 0, 0, 0, 0, 0, 0)
          end if
        end do
        flevel0 = .true.
      else

c     --- Full coarsening
c         --- Both RAPx3, RAPy and RAPz on one level
c         This is taken care of by RAPxyz3 to save space
c         First get some scratch space

      call getscs3 (ibuf, ibuf (sizeyi) * ibuf (sizexi+1) * 9)
      aux1offs = ibuf (offscri)
      call getscs3 (ibuf, ibuf (sizeyi+1) * ibuf (sizexi+1) * 9)
      aux2offs = ibuf (offscri)
      call RAPxyz3 (mat3, rbuf ( ibuf (matri + 1)),
     +               ibuf (sizexi), ibuf (sizeyi), ibuf (sizezi),
     +               rbuf (aux1offs), rbuf (aux2offs),
     + ibuf (slacki), ibuf (slacki + 1), ibuf (slacki + 2),
     + ibuf (slacki + 3), ibuf (slacki + 4), ibuf (slacki + 5))
      flevel0 = .false.
      do ilevel = 1, hilevel - 1
      call RAPxyz3 (rbuf ( ibuf (matri + ilevel)),
     +              rbuf ( ibuf (matri + ilevel + 1)),
     + ibuf(sizexi+ilevel),ibuf(sizeyi+ilevel),ibuf (sizezi+ilevel),
     +           rbuf (aux1offs), rbuf (aux2offs), 0, 0, 0, 0, 0, 0)
      end do
      flevel0 = .true.

c     --- free scratch space (twice)

      call freescs3 (ibuf)
      call freescs3 (ibuf)
      end if
      end
