      subroutine RAPx1P3 ( fmcoef0, fmcoef1, fmcoef2,
     +    cmcoef0, cmcoef1, cmcoef2, nxf, nyf,
     +  sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.2    date   01-07-1997
c           correction of inter grid operators cell centered
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   11-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculate RAP matrix with semicoarsening in x-direction
c   centring shall be decided by the parity of nxf.
c   only one plane at a time, and only ony stencil line
c   saving bufferspace.
c   This subroutine must be called 9 times:
c     with arguments (cm is scratch space)
c   fm (1, 1, p000), fm (1, 1, p100), fm (1, 1, p200),
c   cm (1, 1, 0), cm (1, 1, 1), cm (1, 1, 2),
c   fm (1, 1, p010), fm (1, 1, p110), fm (1, 1, p210),
c   cm (1, 1, 3), cm (1, 1, 4), cm (1, 1, 5),
c     .
c     .
c     .
c     .
c   fm (1, 1, p022), fm (1, 1, p122), fm (1, 1, p222),
c   cm (1, 1, 6), cm (1, 1, 7), cm (1, 1, 8),
c   and after every 3 times RAPy1P must be called.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4
      integer nxf, nyf
      double precision
     +fmcoef0(1-sl1:nxf+sl2,1-sl3:nyf+sl4),
     +fmcoef1(1-sl1:nxf+sl2,1-sl3:nyf+sl4),
     +fmcoef2(1-sl1:nxf+sl2,1-sl3:nyf+sl4),
     +cmcoef0 (nxf/2 + 1, nyf),
     +cmcoef1 (nxf/2 + 1, nyf),
     +cmcoef2 (nxf/2 + 1, nyf)

c     cmcoef0   o    Auxiliary array containing coefficients of coarse matrix
c     cmcoef1   o    Auxiliary array containing coefficients of coarse matrix
c     cmcoef2   o    Auxiliary array containing coefficients of coarse matrix
c     fmcoef0   i    Auxiliary array containing coefficients of fine matrix
c     fmcoef1   i    Auxiliary array containing coefficients of fine matrix
c     fmcoef2   i    Auxiliary array containing coefficients of fine matrix
c     nxf       i    Number of grid x points on fine grid
c     nyf       i    Number of grid y points on fine grid
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixf, iyf, ixc, nxc, centring
      double precision aa0, aa1

c     aa0       Interpolation weight in restriction/prolongation
c     aa1       Interpolation weight in restriction/prolongation
c     cellc     Cell centered (module mgpreset)
c     centring  0 = cellcentering, 1 = vertex centering
c     ixc       Counter to number of grid x points on coarse grid
c     ixf       Counter to number of grid x points on fine grid
c     iyf       Counter to number of grid y points on fine grid
c     nxc       Number of grid x points on coarse grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nxc = nxf/2 + 1
      centring = mod (nxf, 2)
      if (centring.eq.CELLC) then

c     --- cell centered

      aa0 = 0.25d0
      aa1 = 0.75d0
      do iyf = 1, nyf

c     --- stencil left coefficient

        cmcoef0 (1, iyf) = 0d0
        do ixc = 2, nxc - 1
          ixf = 2 * ixc - 2
        cmcoef0 (ixc ,iyf) = aa1*fmcoef0 (ixf ,iyf) +
     +   aa0*(fmcoef1 (ixf,iyf) + fmcoef0 (ixf+1,iyf))
        end do
        cmcoef0 (nxc, iyf) = aa1*fmcoef0 (nxf ,iyf) +
     +   aa0*(fmcoef1 (nxf,iyf))

c       --- stencil center coefficient

          cmcoef1 (1 ,iyf) =
     +                  aa1*fmcoef0 (1,iyf) + aa0*fmcoef2 (1 ,iyf) +
     +                  aa1*fmcoef1 (1,iyf)

        do ixc = 2, nxc - 1
          ixf = 2 * ixc - 2
          cmcoef1 (ixc ,iyf) = aa0*fmcoef0 (ixf ,iyf) +
     +                  aa1*(fmcoef1 (ixf,iyf) + fmcoef0 (ixf+1,iyf))
     +                       + aa0*fmcoef2 (ixf+1 ,iyf) +
     +                  aa1*(fmcoef2 (ixf,iyf) + fmcoef1 (ixf+1,iyf))
        end do

          cmcoef1 (nxc ,iyf) = aa0*fmcoef0 (nxf ,iyf) +
     +                  aa1*fmcoef1 (nxf,iyf) + aa1*fmcoef2 (nxf,iyf)

c         --- end stencil center coefficient

c             --- stencil right coefficient

        cmcoef2 (1,iyf) = aa1*fmcoef2 (1,iyf) +
     +                 aa0* fmcoef1 (1,iyf)
        do ixc = 2, nxc - 1
          ixf = 2 * ixc - 2
          cmcoef2 (ixc,iyf) = aa1*fmcoef2 (ixf+1,iyf) +
     +                 aa0*(fmcoef2 (ixf,iyf) + fmcoef1 (ixf+1,iyf))
        end do
        cmcoef2 (nxc, iyf) = 0d0

c       --- end stencil right coefficient

      end do

c     --- end case cell centered

      else

c     --- begin case vertex centered

      aa0 = 0.25d0
      aa1 = 0.5d0

      do iyf = 1, nyf

c     --- region left boundary

        cmcoef0 (1,iyf) = 0d0
        cmcoef1 (1,iyf) = fmcoef1 (1,iyf) +
     +             aa1*(fmcoef2 (1,iyf) +fmcoef0 (2,iyf))+
     +             aa0*fmcoef1 (2,iyf)
        cmcoef2 (1,iyf) = aa1* (fmcoef2 (2,iyf) +fmcoef2 (1,iyf)) +
     +             aa0*fmcoef1 (2,iyf)
        do ixc = 2,nxc-1
          ixf = 2*ixc - 1
          cmcoef0 (ixc,iyf)= aa1*(fmcoef0 (ixf-1,iyf)
     +           +fmcoef0 (ixf,iyf)) + aa0*fmcoef1 (ixf-1,iyf)
          cmcoef1 (ixc,iyf) = fmcoef1 (ixf,iyf) +
     +            aa1*(fmcoef0 (ixf,iyf) +fmcoef2 (ixf-1,iyf))+
     +            aa1*(fmcoef2 (ixf,iyf) +fmcoef0 (ixf+1,iyf))+
     +            aa0*fmcoef1 (ixf-1,iyf) + aa0*fmcoef1 (ixf+1,iyf)
          cmcoef2 (ixc,iyf) = aa1* (fmcoef2 (ixf +1,iyf)
     +            +fmcoef2 (ixf,iyf)) +aa0*fmcoef1 (ixf+1,iyf)
        end do

c       --- region right boundary

        nxf = 2*nxc - 1
        cmcoef0(nxc,iyf)=aa1*(fmcoef0(nxf-1,iyf)
     +         +fmcoef0(nxf,iyf))+ aa0*fmcoef1(nxf-1,iyf)
        cmcoef1(nxc,iyf)=fmcoef1(nxf,iyf)+
     +  aa1*(fmcoef0(nxf,iyf)+fmcoef2(nxf-1,iyf))+
     +  aa0*fmcoef1(nxf-1,iyf)
        cmcoef2(nxc,iyf)=0d0
      end do

c--- end case vertex centered
      end if

      end
