      subroutine linejcz3 (luzmat, res, cor, nxa, nya, nza,
     +  sl1, sl2, sl3, sl4, sl5, sl6)
c ======================================================================
c
c        programmer     Jos van Kan
c        version 1.1    date   04-06-1997
c           isnazification of all variables and subroutine calls
c           all truncated to max 8 characters.
c        version 1.0    date   10-03-1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   Calculates a line jacobi *correction* in z direction.
c   luz contains the lu decomposition of the "diagonal" blocks
c   D_z c = b - A x_n = r_n
c **********************************************************************
c
c                       KEYWORDS
c
c  multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4, sl5, sl6
      integer nxa, nya, nza
      double precision
     +cor (1-sl1:nxa+sl2, 1-sl3:nya+sl4,
     +1-sl5:nza+sl6),
     +res(1-sl1:nxa+sl2,1-sl3:nya+sl4,
     +1-sl5:nza+sl6),
     +luzmat(1-sl1:nxa+sl2,1-sl3:nya+sl4,
     +1-sl5:nza+sl6,3)

c     cor       o    correction vector
c     luzmat    i    LU dec of "diagonal" blocks "z" numbering
c     nxa       i    Number of grid x points on this level
c     nya       i    Number of grid y points on this level
c     nza       i    Number of grid z points on this level
c     res       i    Residual (any level)
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ixa, iya, iza

c     diag      Diagonal index in tridiag matrix (module mgpreset)
c     ixa       Counter to number of grid x points on this level
c     iya       Counter to number of grid y points on this level
c     iza       Counter to number of grid z points on this level
c     sub       Subdiagonal index in tridiagonal matrix (module
c               mgpreset)
c     sup       Superdiagonal index in tridiagonal matrix (module
c               mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c      trivial
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      do iya = 1, nya
        do ixa = 1, nxa

c       --- first forward substitution : L s = b

          cor (ixa, iya, 1) = res (ixa, iya, 1)
        end do
      end do

      do iza = 2, nza
        do iya = 1, nya
          do ixa = 1, nxa
            cor (ixa, iya, iza) = res (ixa, iya, iza) -
     +              cor (ixa, iya, iza-1) * luzmat (ixa, iya, iza, SUB)
          end do
        end do
      end do

c     --- then backward substitution Ux = s

      do iya = 1, nya
        do ixa = 1, nxa
          cor (ixa, iya, nza) = cor (ixa, iya, nza) /
     +         luzmat (ixa, iya, nza, DIAG)
        end do
      end do

      do iza = nza - 1, 1, -1
        do iya = 1, nya
          do ixa = 1, nxa
            cor (ixa, iya, iza) = (cor (ixa, iya, iza) -
     +          cor (ixa, iya, iza + 1) * luzmat (ixa, iya, iza, SUP))
     +          /luzmat (ixa, iya, iza, DIAG)
          end do
        end do
      end do
      end
