      subroutine pmap319 ( maparr )
c ======================================================================
c
c        programmer     Jos van Kan
c       version 1.2    date  8- 7-1997
c       version 1.1    10-04-1997
c        version 1.0    date Tue Apr  8 1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   pmap319 serves to change the default mapping of the 19-point laplacian
c   molecule. The user can change the order by giving a permutation
c   in the 19-entry array pmap of the vector 1..19.
c   The points [02][02][02] are zero coefficients. They get a default
c   mapping 20..27
c   The default mapping is:
c   0-z plane
c             22  5 23
c              2  3  4
c             20  1 21
c  1-z plane
c              12 13 14
c               9 10 11
c               6  7  8
c 2-z plane
c              26 19 27
c              16 17 18
c              24 15 25
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maparr (27)

c     maparr    i    Contains permutation for p00 ... p22 (1..9 by
c                          default)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, perm (27)

c     i         General loop variable
c     molstruc  To signify the molecule structure on the finest level:
c               pt27 (default), pt19 or pt7 (module mgpreset)
c     p000      Index of point in stencil 3D (module mgpreset)
c     p001      Index of point in stencil 3D (module mgpreset)
c     p002      Index of point in stencil 3D (module mgpreset)
c     p010      Index of point in stencil 3D (module mgpreset)
c     p011      Index of point in stencil 3D (module mgpreset)
c     p012      Index of point in stencil 3D (module mgpreset)
c     p020      Index of point in stencil 3D (module mgpreset)
c     p021      Index of point in stencil 3D (module mgpreset)
c     p022      Index of point in stencil 3D (module mgpreset)
c     p100      Index of point in stencil 3D (module mgpreset)
c     p101      Index of point in stencil 3D (module mgpreset)
c     p102      Index of point in stencil 3D (module mgpreset)
c     p110      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p112      Index of point in stencil 3D (module mgpreset)
c     p120      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     p122      Index of point in stencil 3D (module mgpreset)
c     p200      Index of point in stencil 3D (module mgpreset)
c     p201      Index of point in stencil 3D (module mgpreset)
c     p202      Index of point in stencil 3D (module mgpreset)
c     p210      Index of point in stencil 3D (module mgpreset)
c     p211      Index of point in stencil 3D (module mgpreset)
c     p212      Index of point in stencil 3D (module mgpreset)
c     p220      Index of point in stencil 3D (module mgpreset)
c     p221      Index of point in stencil 3D (module mgpreset)
c     p222      Index of point in stencil 3D (module mgpreset)
c     perm      Permutation vector
c     pt19      Named constant (1). Molecule has 19 points on level 0
c               (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     MGEXIT3   User supplied routine to exit the package with an error
c               condition. Default: stop. (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- sanity check
c         we check whether pmap is a permutation
c         if not an error is signalled

      do i = 1, 19
      perm (i) = 0
      end do

      do i = 1, 19
      perm (maparr (i)) = 1
      end do

      do i = 1, 19
      if (perm (i).eq.0) then
      write (*, *) "Fatal MG error in pmap: map array is not a ",
     +             "permutation. Program halted."
      call mgexit3
      end if
      end do

      p000 = 20
      p100 = maparr (1)
      p200 = 21
      p010 = maparr (2)
      p110 = maparr (3)
      p210 = maparr (4)
      p020 = 22
      p120 = maparr (5)
      p220 = 23
      p001 = maparr (6)
      p101 = maparr (7)
      p201 = maparr (8)
      p011 = maparr (9)
      p111 = maparr (10)
      p211 = maparr (11)
      p021 = maparr (12)
      p121 = maparr (13)
      p221 = maparr (14)
      p002 = 24
      p102 = maparr (15)
      p202 = 25
      p012 = maparr (16)
      p112 = maparr (17)
      p212 = maparr (18)
      p022 = 26
      p122 = maparr (19)
      p222 = 27
      molstruc = PT19
      end
