      subroutine pmap37 ( maparr )
c ======================================================================
c
c        programmer     Jos van Kan
c       version 1.2    date  8- 7-1997
c       version 1.1    10-04-1997
c        version 1.0    date Tue Apr  8 1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   pmap37 serves to change the default mapping of the 7-point laplacian
c   molecule. The user can change the order by giving a permutation
c   in the 7-entry array pmap of the vector 1..7.
c   The points *11, 1*1, 11* are the only non-zero coefficients.
c   The rest gets a default mapping 9..27
c   The default mapping is:
c   0-z plane
c              13 14 15
c              11  1 12
c               8  9 10
c  1-z plane
c              18  6 19
c               3  4  5
c              16  2 17
c  2-z plane
c              25 26 27
c              23  7 24
c              20 21 22
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maparr (7)

c     maparr    i    Contains permutation for p00 ... p22 (1..9 by
c                          default)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, perm (7)

c     i         General loop variable
c     molstruc  To signify the molecule structure on the finest level:
c               pt27 (default), pt19 or pt7 (module mgpreset)
c     p000      Index of point in stencil 3D (module mgpreset)
c     p001      Index of point in stencil 3D (module mgpreset)
c     p002      Index of point in stencil 3D (module mgpreset)
c     p010      Index of point in stencil 3D (module mgpreset)
c     p011      Index of point in stencil 3D (module mgpreset)
c     p012      Index of point in stencil 3D (module mgpreset)
c     p020      Index of point in stencil 3D (module mgpreset)
c     p021      Index of point in stencil 3D (module mgpreset)
c     p022      Index of point in stencil 3D (module mgpreset)
c     p100      Index of point in stencil 3D (module mgpreset)
c     p101      Index of point in stencil 3D (module mgpreset)
c     p102      Index of point in stencil 3D (module mgpreset)
c     p110      Index of point in stencil 3D (module mgpreset)
c     p111      Index of point in stencil 3D (module mgpreset)
c     p112      Index of point in stencil 3D (module mgpreset)
c     p120      Index of point in stencil 3D (module mgpreset)
c     p121      Index of point in stencil 3D (module mgpreset)
c     p122      Index of point in stencil 3D (module mgpreset)
c     p200      Index of point in stencil 3D (module mgpreset)
c     p201      Index of point in stencil 3D (module mgpreset)
c     p202      Index of point in stencil 3D (module mgpreset)
c     p210      Index of point in stencil 3D (module mgpreset)
c     p211      Index of point in stencil 3D (module mgpreset)
c     p212      Index of point in stencil 3D (module mgpreset)
c     p220      Index of point in stencil 3D (module mgpreset)
c     p221      Index of point in stencil 3D (module mgpreset)
c     p222      Index of point in stencil 3D (module mgpreset)
c     perm      Permutation vector
c     pt7       Named constant (2). Molecule has 7 points on level 0
c               (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     MGEXIT3   User supplied routine to exit the package with an error
c               condition. Default: stop. (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- sanity check
c         we check whether pmap is a permutation
c         if not an error is signalled

      do i = 1, 7
      perm (i) = 0
      end do

      do i = 1, 7
      perm (maparr (i)) = 1
      end do

      do i = 1, 7
      if (perm (i).eq.0) then
      write (*, *) "Fatal MG error in pmap: map array is not a ",
     +             "permutation. Program halted."
      call mgexit3
      end if
      end do

      p000 =  8
      p100 =  9
      p200 = 10
      p010 = 11
      p110 = maparr (1)
      p210 = 12
      p020 = 13
      p120 = 14
      p220 = 15
      p001 = 16
      p101 = maparr (2)
      p201 = 17
      p011 = maparr (3)
      p111 = maparr (4)
      p211 = maparr (5)
      p021 = 18
      p121 = maparr (6)
      p221 = 19
      p002 = 20
      p102 = 21
      p202 = 22
      p012 = 23
      p112 = maparr (7)
      p212 = 24
      p022 = 25
      p122 = 26
      p222 = 27
      molstruc = pt7
      end
