      subroutine prelude3 ( mat3, nx, ny, nz, acorsnin, ahilevel,
     +   rbuf, bufsize,
     +   sl1, sl2, sl3, sl4, sl5, sl6,
     +   ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c       version 1.2    date  8- 7-1997
c       version 1.1    10-04-1997
c        version 1.0    date Tue Apr  8 1997
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c   pilotcode 3d mgpackage
c
c **********************************************************************
c
c                       MODULES USED
c
      use mgpreset
c **********************************************************************
c
c                       DESCRIPTION
c
c   In the prelude all common blocks will be set, all
c   memory allocated, the RAP for all levels made, together
c   with the LU decomposition of the block tridiagonal
c   mat3rices that form the diagonal of A on all grids.
c   There are basically two methods:
c   1. Full coarsening
c   2. semi coarsening
c   Semi coarsening is more robust, full coarsening may be quicker
c   For a description of all considerations see mg2.readme
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      double precision mat3 (*), rbuf (*)
      integer ibuf (*)
      integer bufsize
      integer nx, ny, nz, acorsnin, ahilevel,
     +    sl1, sl2, sl3, sl4, sl5, sl6

c     acorsnin  i    1=full coarsening 2=semicoarsening  (formal parameter)
c     ahilevel  i    Maximum level of coarse grids (level 0 is finest)
c                    (formal parameter)
c     bufsize   i    Size of buffer needed to accommodate all coarse grid
c                    quantities
c     ibuf      o    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat3      i    The nx x ny x nz x 27  coefficient matrix of the
c                    discrete Laplacian
c     nx        i    Number of grid x points on finest grid
c     ny        i    Number of grid y points on finest grid
c     nz        i    Number of points in the z-direction
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c     sl5       i    Slack variable, z lower bound
c     sl6       i    Slack variable, z upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c     corsnini  Ibuf (corsnini) contains the method of coarsening
c               for this block. (fullcors or semicors) (module
c               mgpreset)
c     errcnti   Ibuf (errcnti) contains number of times the error
c               handler has been called for this block. (module
c               mgpreset)
c     hileveli  Ibuf (hileveli) contains the highest level of
c               coarsening for this block. (<= MAXLEVEL) (module
c               mgpreset)
c     maxlevel  Maximum level allowed for (semi)coarsening (module
c               mgpreset)
c     maxrbufi  Ibuf (maxrbufi) contains maximum size of rbuf (module
c               mgpreset)
c     offrbufi  Ibuf (offrbufi) contains pointer to first unused place
c               in rbuf (module mgpreset)
c     offscri   Ibuf (offscri) contains index where scratch space begins
c               in rbuf (module mgpreset)
c     prelud_i  Ibuf (prelud_i) = 100 if prelude has been called
c               for this block. (module mgpreset)
c     sizexi    Ibuf(sizexi + ilevel)) contains number of points
c               in x direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizeyi    Ibuf(sizeyi + ilevel)) contains number of points
c               in y direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     sizezi    Ibuf(sizezi + ilevel)) contains number of points
c               in z direction at level ilevel. (0 = finest) (module
c               mgpreset)
c     slacki    Ibuf (slacki) is the first element of the 6 element
c               slack array xslacklo, xslackhi, yslacklo, yslackhi,
c               zslacklo, zslackhi, (module mgpreset)
c     wirecnti  Ibuf (wirecnti) contains the wirecount (module
c               mgpreset)
c     yes       Named constant (module mgpreset)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ERRMSG3   Reports and counts errors (MG3)
c     LU_ALL3   Block LU decomposition on all levels, both
c               directions (MG3)
c     MKLVINF3  Reserves space for all matrices and arrays (MG3)
c     RAP_ALL3  Froms coarse grid operator on all levels (MG3)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- sanity check

      if (ahilevel.gt.MAXLEVEL) then
         call errmsg3 ( "prelude", "hilevel exceeds MAXLEVEL", .FALSE.,
     +   ibuf)
      end if

c     --- Set Buf parms

      ibuf (MAXRBUFi) = bufsize
      ibuf (offrbufi) = 1
      ibuf (wirecnti) = 0
      ibuf (offscri) = ibuf (MAXRBUFi) + 1

c     --- set params for this block

      ibuf (hileveli) = ahilevel
      ibuf (corsnini) = acorsnin
      ibuf (sizexi) = nx
      ibuf (sizeyi) = ny
      ibuf (sizezi) = nz
      ibuf (slacki) = sl1
      ibuf (slacki + 1) = sl2
      ibuf (slacki + 2) = sl3
      ibuf (slacki + 3) = sl4
      ibuf (slacki + 4) = sl5
      ibuf (slacki + 5) = sl6

c     --- set error count

      ibuf (errcnti) = 0

      call mklvinf3 (ibuf)
      call RAP_all3 (mat3, rbuf, ibuf)
      call LU_all3 (mat3, rbuf, ibuf)
      ibuf (prelud_i) = YES
      end
